@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0+
 /*
- * Copyright 2011-2015 Red Hat, Inc.
- * Copyright 2011 Giovanni Campagna <scampa.giovanni@gmail.com>
+ * Copyright (C) 2011 - 2015 Red Hat, Inc.
+ * Copyright (C) 2011 Giovanni Campagna <scampa.giovanni@gmail.com>
  */
 
 /**
@@ -302,7 +302,7 @@ add_wireless_secrets (RequestData *request,
 	const char *key_mgmt = nm_setting_wireless_security_get_key_mgmt (s_wsec);
 	NMSecretAgentSimpleSecret *secret;
 
-	if (!key_mgmt)
+	if (!key_mgmt || nm_streq (key_mgmt, "owe"))
 		return FALSE;
 
 	if (NM_IN_STRSET (key_mgmt, "wpa-psk", "sae")) {
