@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1+
 /*
- * Copyright 2012 - 2013 Red Hat, Inc.
+ * Copyright (C) 2012 - 2013 Red Hat, Inc.
  */
 
 #include "nm-default.h"
@@ -34,10 +34,10 @@ NM_GOBJECT_PROPERTIES_DEFINE (NMSettingBridgePort,
 );
 
 typedef struct {
+	GPtrArray *vlans;
 	guint16 priority;
 	guint16 path_cost;
-	gboolean hairpin_mode;
-	GPtrArray *vlans;
+	bool hairpin_mode:1;
 } NMSettingBridgePortPrivate;
 
 G_DEFINE_TYPE (NMSettingBridgePort, nm_setting_bridge_port, NM_TYPE_SETTING)
@@ -439,6 +439,9 @@ nm_setting_bridge_port_init (NMSettingBridgePort *setting)
 	NMSettingBridgePortPrivate *priv = NM_SETTING_BRIDGE_PORT_GET_PRIVATE (setting);
 
 	priv->vlans = g_ptr_array_new_with_free_func ((GDestroyNotify) nm_bridge_vlan_unref);
+
+	priv->priority = NM_BR_PORT_DEF_PRIORITY;
+	priv->path_cost = NM_BR_PORT_DEF_PATH_COST;
 }
 
 /**
@@ -497,7 +500,6 @@ nm_setting_bridge_port_class_init (NMSettingBridgePortClass *klass)
 	    g_param_spec_uint (NM_SETTING_BRIDGE_PORT_PRIORITY, "", "",
 	                       0, NM_BR_PORT_MAX_PRIORITY, NM_BR_PORT_DEF_PRIORITY,
 	                       G_PARAM_READWRITE |
-	                       G_PARAM_CONSTRUCT |
 	                       NM_SETTING_PARAM_INFERRABLE |
 	                       G_PARAM_STATIC_STRINGS);
 
@@ -517,9 +519,8 @@ nm_setting_bridge_port_class_init (NMSettingBridgePortClass *klass)
 	 */
 	obj_properties[PROP_PATH_COST] =
 	    g_param_spec_uint (NM_SETTING_BRIDGE_PORT_PATH_COST, "", "",
-	                       0, NM_BR_PORT_MAX_PATH_COST, 100,
+	                       0, NM_BR_PORT_MAX_PATH_COST, NM_BR_PORT_DEF_PATH_COST,
 	                       G_PARAM_READWRITE |
-	                       G_PARAM_CONSTRUCT |
 	                       G_PARAM_STATIC_STRINGS);
 
 	/**
@@ -573,17 +574,10 @@ nm_setting_bridge_port_class_init (NMSettingBridgePortClass *klass)
 	                        G_PARAM_READWRITE |
 	                        NM_SETTING_PARAM_INFERRABLE |
 	                        G_PARAM_STATIC_STRINGS);
-
-	_properties_override_add_override (properties_override,
-	                                   obj_properties[PROP_VLANS],
-	                                   G_VARIANT_TYPE ("aa{sv}"),
-	                                   _nm_utils_bridge_vlans_to_dbus,
-	                                   _nm_utils_bridge_vlans_from_dbus,
-	                                   NULL);
+	_nm_properties_override_gobj (properties_override, obj_properties[PROP_VLANS], &nm_sett_info_propert_type_bridge_vlans);
 
 	g_object_class_install_properties (object_class, _PROPERTY_ENUMS_LAST, obj_properties);
 
 	_nm_setting_class_commit_full (setting_class, NM_META_SETTING_TYPE_BRIDGE_PORT,
 	                               NULL, properties_override);
-
 }
