@@ -35,18 +35,18 @@ NM_GOBJECT_PROPERTIES_DEFINE_BASE (
 
 typedef struct {
 	char *parent;
-	NMIPTunnelMode mode;
 	char *local;
 	char *remote;
-	guint ttl;
-	guint tos;
-	gboolean path_mtu_discovery;
 	char *input_key;
 	char *output_key;
+	guint ttl;
+	guint tos;
 	guint encapsulation_limit;
 	guint flow_label;
+	NMIPTunnelMode mode;
 	guint32 mtu;
 	guint32 flags;
+	bool path_mtu_discovery:1;
 } NMSettingIPTunnelPrivate;
 
 G_DEFINE_TYPE (NMSettingIPTunnel, nm_setting_ip_tunnel, NM_TYPE_SETTING)
@@ -291,14 +291,6 @@ nm_setting_ip_tunnel_get_flags (NMSettingIPTunnel *setting)
 
 /*****************************************************************************/
 
-gboolean
-_nm_ip_tunnel_mode_is_layer2 (NMIPTunnelMode mode)
-{
-	return NM_IN_SET (mode,
-	                  NM_IP_TUNNEL_MODE_GRETAP,
-	                  NM_IP_TUNNEL_MODE_IP6GRETAP);
-}
-
 static gboolean
 verify (NMSetting *setting, NMConnection *connection, GError **error)
 {
@@ -349,7 +341,7 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 		return FALSE;
 	}
 
-	if (priv->local && !nm_utils_ipaddr_valid (family, priv->local)) {
+	if (priv->local && !nm_utils_ipaddr_is_valid (family, priv->local)) {
 		g_set_error (error,
 		             NM_CONNECTION_ERROR,
 		             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -369,7 +361,7 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 		return FALSE;
 	}
 
-	if (!nm_utils_ipaddr_valid (family, priv->remote)) {
+	if (!nm_utils_ipaddr_is_valid (family, priv->remote)) {
 		g_set_error (error,
 		             NM_CONNECTION_ERROR,
 		             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -452,7 +444,9 @@ verify (NMSetting *setting, NMConnection *connection, GError **error)
 	}
 
 	if (   nm_connection_get_setting_wired (connection)
-	    && !_nm_ip_tunnel_mode_is_layer2 (priv->mode)) {
+	    && !NM_IN_SET (priv->mode,
+	                   NM_IP_TUNNEL_MODE_GRETAP,
+	                   NM_IP_TUNNEL_MODE_IP6GRETAP)) {
 		g_set_error (error,
 		             NM_CONNECTION_ERROR,
 		             NM_CONNECTION_ERROR_INVALID_PROPERTY,
@@ -582,8 +576,11 @@ set_property (GObject *object, guint prop_id,
 /*****************************************************************************/
 
 static void
-nm_setting_ip_tunnel_init (NMSettingIPTunnel *setting)
+nm_setting_ip_tunnel_init (NMSettingIPTunnel *self)
 {
+	NMSettingIPTunnelPrivate *priv = NM_SETTING_IP_TUNNEL_GET_PRIVATE (self);
+
+	priv->path_mtu_discovery = TRUE;
 }
 
 /**
@@ -703,7 +700,6 @@ nm_setting_ip_tunnel_class_init (NMSettingIPTunnelClass *klass)
 	    g_param_spec_uint (NM_SETTING_IP_TUNNEL_TTL, "", "",
 	                       0, 255, 0,
 	                       G_PARAM_READWRITE |
-	                       G_PARAM_CONSTRUCT |
 	                       NM_SETTING_PARAM_INFERRABLE |
 	                       G_PARAM_STATIC_STRINGS);
 
@@ -719,7 +715,6 @@ nm_setting_ip_tunnel_class_init (NMSettingIPTunnelClass *klass)
 	    g_param_spec_uint (NM_SETTING_IP_TUNNEL_TOS, "", "",
 	                       0, 255, 0,
 	                       G_PARAM_READWRITE |
-	                       G_PARAM_CONSTRUCT |
 	                       NM_SETTING_PARAM_INFERRABLE |
 	                       G_PARAM_STATIC_STRINGS);
 
@@ -734,7 +729,6 @@ nm_setting_ip_tunnel_class_init (NMSettingIPTunnelClass *klass)
 	    g_param_spec_boolean (NM_SETTING_IP_TUNNEL_PATH_MTU_DISCOVERY, "", "",
 	                          TRUE,
 	                          G_PARAM_READWRITE |
-	                          G_PARAM_CONSTRUCT |
 	                          NM_SETTING_PARAM_INFERRABLE |
 	                          G_PARAM_STATIC_STRINGS);
 
@@ -780,7 +774,6 @@ nm_setting_ip_tunnel_class_init (NMSettingIPTunnelClass *klass)
 	    g_param_spec_uint (NM_SETTING_IP_TUNNEL_ENCAPSULATION_LIMIT, "", "",
 	                       0, 255, 0,
 	                       G_PARAM_READWRITE |
-	                       G_PARAM_CONSTRUCT |
 	                       NM_SETTING_PARAM_INFERRABLE |
 	                       G_PARAM_STATIC_STRINGS);
 
@@ -796,7 +789,6 @@ nm_setting_ip_tunnel_class_init (NMSettingIPTunnelClass *klass)
 	    g_param_spec_uint (NM_SETTING_IP_TUNNEL_FLOW_LABEL, "", "",
 	                       0, (1 << 20) - 1, 0,
 	                       G_PARAM_READWRITE |
-	                       G_PARAM_CONSTRUCT |
 	                       NM_SETTING_PARAM_INFERRABLE |
 	                       G_PARAM_STATIC_STRINGS);
 
@@ -812,7 +804,6 @@ nm_setting_ip_tunnel_class_init (NMSettingIPTunnelClass *klass)
 	    g_param_spec_uint (NM_SETTING_IP_TUNNEL_MTU, "", "",
 	                       0, G_MAXUINT32, 0,
 	                       G_PARAM_READWRITE |
-	                       G_PARAM_CONSTRUCT |
 	                       NM_SETTING_PARAM_FUZZY_IGNORE |
 	                       G_PARAM_STATIC_STRINGS);
 
