@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1+
 /*
- * Copyright 2011 - 2012 Red Hat, Inc.
+ * Copyright (C) 2011 - 2012 Red Hat, Inc.
  */
 
 #ifndef __NM_DEVICE_INFINIBAND_H__
@@ -27,20 +27,13 @@ G_BEGIN_DECLS
 /**
  * NMDeviceInfiniband:
  */
-struct _NMDeviceInfiniband {
-	NMDevice parent;
-};
-
-typedef struct {
-	NMDeviceClass parent;
-
-	/*< private >*/
-	gpointer padding[4];
-} NMDeviceInfinibandClass;
+typedef struct _NMDeviceInfinibandClass NMDeviceInfinibandClass;
 
 GType nm_device_infiniband_get_type (void);
 
+NM_DEPRECATED_IN_1_24_FOR (nm_device_get_hw_address)
 const char * nm_device_infiniband_get_hw_address (NMDeviceInfiniband *device);
+
 gboolean     nm_device_infiniband_get_carrier (NMDeviceInfiniband *device);
 
 G_END_DECLS
