@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: LGPL-2.1+
 /*
- * Copyright 2007 - 2008 Novell, Inc.
- * Copyright 2008 - 2014 Red Hat, Inc.
+ * Copyright (C) 2007 - 2008 Novell, Inc.
+ * Copyright (C) 2008 - 2014 Red Hat, Inc.
  */
 
 #ifndef __NM_IP6_CONFIG_H__
@@ -23,13 +23,8 @@
 /**
  * NMIP6Config:
  */
-typedef struct {
-	NMIPConfig parent;
-} NMIP6Config;
-
-typedef struct {
-	NMIPConfigClass parent;
-} NMIP6ConfigClass;
+typedef struct _NMIP6Config NMIP6Config;
+typedef struct _NMIP6ConfigClass NMIP6ConfigClass;
 
 GType nm_ip6_config_get_type (void);
 
