@@ -1,6 +1,5 @@
 // SPDX-License-Identifier: GPL-2.0+
-/* nm-platform.c - Handle runtime kernel networking configuration
- *
+/*
  * Copyright (C) 2015 Red Hat, Inc.
  */
 
@@ -502,7 +501,7 @@ test_recv (TestRecvFixture *fixture, gconstpointer user_data)
 	data->check (loop, listener);
 
 	nm_clear_g_source (&sd_id);
-	g_clear_pointer (&loop, g_main_loop_unref);
+	nm_clear_pointer (&loop, g_main_loop_unref);
 }
 
 static void
